#ifndef bqtStdinHH
#define bqtStdinHH

#include <vector>

using std::vector;

#include "buffer.hh"
#include "datapacket.hh"
#include "protocols.hh"

/* These messages are defined:

    Open(ConnID, Protocol, Port)       (Beacon only)
    Data(ConnID, Length, Data[])       (Beacon, master)
    Close(ConnID)                      (Beacon, master)
    Listen(Protocol, Port)             (Master only)
  
   Both TCP and UDP use the same way: start with
   NewConnection, use Data, end with ConnectionEnd.
*/

class StdinType
{
public:
	StdinType();
	~StdinType();

	bool WantSpeak() const;
    void Speak(int fd);
    
    bool IsDead() const;
    void Die();

    DataPacket Read(int fd);

    /* Called by a communication entity */
    void SendPacket(const struct DataPacket &pak);

	void AssignConnID(unsigned id, const Connection &conn);
    
private:
	friend class ConnectionCache;
    /* Called by StdinType's ConnectionCache */
    void AddOpeningMsg(unsigned ConnID, SockProtocol proto, int port);
    void AddClosingMsg(unsigned ConnID);
    
    void AddDataMsg(unsigned ConnID, const vector<Datatype> &data);
    void AddListenMsg(SockProtocol proto, int port);
    void AddMsg(const struct MasterMsg &m);

private:
	class ConnectionCache *cache;

    /* Queue of bytes to be sent */
    Buffer SendQueue;
    
    unsigned ExpectLen, ExpectGot;
    Datatype *ExpectBuf;
};

#endif
